*
* HTX-242 Microphone Control Protocol Implementation for the 8051
*
* Copyright 1997-2000 Dave Dunfield
* All rights reserved.
*
* Setup Parameters
* BAUD	= (crystal / baud-rate) / 384
* 3	= (11059200 / 9600) / 384
BAUD	EQU	3		Baudrate divisor (9600) - Normal
;BAUD	EQU	1		Baudrate divisor (28800) - For EMILY52
*
* I/O Lines (To HTX-242 Microphone connector)
*
DATA	BIT	P1.0		Data line to radio
CLK	BIT	P1.1		Clock line from radio
CON	BIT	P1.2		Control line to radio
PTT	BIT	P1.3		PTT (0 to xmit)
BUSY	BIT	P1.4		BUSY (0 = Busy)
*
* HTX-242 Microphone -> Radio codes
* Digits 1 to 9 are transmitted as $01 to $09
*
ZERO	EQU	$0A		Zero Digit
STAR	EQU	$0B		Star (asterix)
POUND	EQU	$0C		Pound sign (Octothorpe)
_A	EQU	$0D		A Digit
_B	EQU	$0E		B Digit
_C	EQU	$0F		C Digit
_D	EQU	$10		D Digit
CALL	EQU	$11		CALL channel
VFO	EQU	$12		VFO Select (A/B)
MR	EQU	$13		MEMORY recall
MHZ	EQU	$14		Mhz select
UP	EQU	$15		UP (Step/Mhz/Channel)
DOWN	EQU	$16		DOWN (Step/Mhz/Channel)
*
	ORG	$0800		Code goes here
* I/O & Stack Initializations
	CLR	CON		Control line to zero
	MOV	SP,#7		Reset SP
* Setup timer 1 for baud rate generation
	MOV	TMOD,#%00100000	T1=8 bit auto-reload
	MOV	TH1,#-BAUD	Timer-1 reload value
	MOV	TL1,#-BAUD	Timer-1 initial value
	MOV	TCON,#%01001001	Run 1, Hold 0
* Initialize the serial port
	MOV	SCON,#%01010010	Mode 1, REN, TXRDY, RXEMPTY
*
* Flush the receiver (Wait till all chars received)
*
FLUSH	MOV	R0,#0		Delay count
flush1	JNB	SCON.0,flush2	No data
	CLR	SCON.0		Reset it
	MOV	A,SBUF		Read data
	SJMP	FLUSH		Restart
flush2	DJNZ	R0,flush1	Wait for expiry
*
* Wait for char & execute command
*
WAITCMD	CLR	BUSY		Indicate ready for command
	ACALL	GETCHR		Get a character
	SETB	BUSY		Indicate busy
	CJNE	A,#'t',waitc1	Not 'T'ransmit
	CLR	PTT		Set PTT on
	SJMP	success		Next command
waitc1	CJNE	A,#'r',waitc2	Not 'R'eceive
	SETB	PTT		Set PTT off
	SJMP	success		Next command
waitc2	MOV	B,A		Save for later
	MOV	DPTR,#CODETAB	Point to code table
waitc3	CLR	A		Zero offset
	MOVC	A,[A+DPTR]	Get char from table
	INC	DPTR		Advance to next
	JZ	badcmd		Invalid command
	CJNE	A,B,waitc3	No match, try next
* We have found the character to send
	MOV	A,DPL		Get LOW value
	ADD	A,#-CODETAB	Adjust to binary code
	ACALL	SEND		Send to radio
	JC	comerr		Communications error
success	ACALL	DELAY		Allow it to finish
	MOV	A,#'*'		Indicate success
reperr	ACALL	WRCHR		Write character
	AJMP	waitcmd		And get next command
badcmd	MOV	A,#'?'		Indicate Invalid command
	SJMP	reperr		Report & get next
comerr	MOV	A,#'$'		Indicate communication error
	SJMP	reperr		Report & get next
*
* Command to code translation table
*
CODETAB	STRZ	'1234567890*#ABCDcvmhud'
*
* Send a 5-bit code (ACC) to the radio
* On exit, Carry flag (C) indicates error, (NC=Ok)
*
SEND	PUSH	A		Save ACC
	PUSH	0		Save R0
	RLC	A		Drop unused...
	RLC	A		Upper three...
	RLC	A		Bits of code.
	SETB	CON		Set control line
	ACALL	send4		Wait for clock
	JNC	send8		Incorrect lead-in (Should be 1)
	ACALL	send4		Wait for clock
	JNC	send8		Incorrect lead-in (Should be 1)
	ACALL	send4		Wait for clock
	JC	send8		Incorrect lead-in (Should be 0)
	MOV	R0,#5		Send 5 bits
send1	RLC	A		Carry = Acc.0
	JNB	CLK,*		Wait for clock
	MOV	DATA,C		Write to data
	JB	CLK,*		Wait for clock away
	DJNZ	R0,send1	Next bit
	DJNZ	R0,*		Extra time for last bit
	SETB	DATA		Release data line again
	MOV	A,100		Following delay
send3	DJNZ	R0,*		Decrement inner loop
	DEC	A		Reduce count
	JNZ	send3		Do them all
* Release control line & exit with success
	CLR	CON		Release line
	POP	0		Restore R0
	POP	A		Restore ACC
	CLR	C		Indicate success
	RET
* Wait for clock bit & sample line (into C)
send4	MOV	R0,#0		Time to wait
send5	JB	CLK,send6	We have bit
	DJNZ	R0,send5	Keep looking
	SJMP	send7		Didn't find
send6	JNB	CLK,send9	Bit has gone away
	DJNZ	R0,send6	Keep looking
* Clock was not found - remove return address & exit/fail
send7	POP	A		Clean stack
	POP	A		Clean stack
* An error has occured - exit/fail
send8	CLR	CON		Release data line
	POP	0		Restore R0
	POP	A		Restore ACC
	SETB	C		Indicate error
	RET
* Clock found - sample data line
send9	MOV	C,DATA		Read data line
	RET
*
* Generate a short delay
*
DELAY	PUSH	0		Save R0
	PUSH	1		Save R1
	MOV	R1,#0		Initial outer count
delay1	MOV	R0,#0		Setup inner count
	DJNZ	R0,*		Decrement inner count
	DJNZ	R1,delay1	Decrement outer count
	POP	1		Restore R1
	POP	0		Restore R0
	RET
*
* Write a character to the serial port
*
WRCHR	JNB	SCON.1,*	Wait for the bit
	CLR	SCON.1		Indicte we are sending
	MOV	SBUF,A		Write out char
	RET
*
* Get a character from the serial port
*
GETCHR	JNB	SCON.0,GETCHR	Wait for it
GETC1	CLR	SCON.0		Indicate we received
	MOV	A,SBUF		Read the data
	RET
